function varargout = minEA(...
    evaluator, evalparams, ...
    initializer, iniparams, ...
    selector, selparams, ...
    crossover, crossparams, ...
    mutator, mutparams, ...
    replacement, repparams, ...
    o)

if ~exist('o','var') | isempty(o), o = struct; end
if iscell(o), o = struct(o{:}); end
% if ~isfield(o,'TolX'), o.TolX = 1e-10; end
% if ~isfield(o,'TolFun'), o.TolFun = 1e-8; end
if ~isfield(o,'MaxFunEvals'), o.MaxFunEvals = 100000; end
if ~isfield(o,'MaxIter'), o.MaxIter = inf; end
if ~isfield(o,'StopFitness'), o.StopFitness = 1e-6; end
if ~isfield(o,'verbose'); o.verbose = 1; end
    
pop = feval(initializer, o, iniparams{:});
f = feval(evaluator, pop, o, evalparams{:});
evals = length(f);

[fmin, imin] = min(f);

hist.gen = 0;
hist.evals = evals;
hist.x = pop(imin,:);
hist.fmin = f(imin);
hist.fmax = max(f);
hist.fmed = median(f);
hist.fmean = mean(f);
hist.fstd = std(f);
hist.nonimpr = 0;

gen = 0;
nonimpr = 0;
while (gen < o.MaxIter) & ...
      (evals < o.MaxFunEvals) & ...
      (fmin > o.StopFitness),
    
    gen = gen + 1;
    if o.verbose,
        fprintf('Generation: %d\n', gen);
    end
    
    iparents = feval(selector, f, o, selparams{:});
    parents = pop(iparents, :);
    
    offspring = feval(crossover, parents, o, crossparams{:});
    offspring = feval(mutator, offspring, o, mutparams{:});
    
    foff = feval(evaluator, offspring, o, evalparams{:});
    evals = evals + length(foff);
    
    [pop, f] = feval(replacement, pop, f, offspring, foff, o, repparams{:});
    [fmin, imin] = min(f);
    
    if fmin < hist.fmin(end),
        hist.gen(end+1) = gen;
        hist.evals(end+1) = evals;
        hist.x(end+1,:) = pop(imin,:);
        hist.fmin(end+1) = f(imin);
        hist.fmax(end+1) = max(f);
        hist.fmed(end+1) = median(f);
        hist.fmean(end+1) = mean(f);
        hist.fstd(end+1) = std(f);
        hist.nonimpr(end+1) = nonimpr;
        nonimpr = 0;
    else
        nonimpr = nonimpr + 1;
    end

end

hist.gen(end+1) = gen;
hist.evals(end+1) = evals;
hist.x(end+1,:) = pop(imin,:);
hist.fmin(end+1) = f(imin);
hist.fmax(end+1) = max(f);
hist.fmed(end+1) = median(f);
hist.fmean(end+1) = mean(f);
hist.fstd(end+1) = std(f);
hist.nonimpr(end+1) = nonimpr;

varargout{1} = hist.x(end,:);
if nargout > 1, varargout{2} = hist.fmin(end); end
if nargout > 2, varargout{3} = hist; end    