% Clear the workspace
clear;

% Set the number of cities
ncities = 30;
% Generate their x,y coordinates randomly in (0,1)x(0,1)
cities = rand(ncities,2);
% Compute distance matrix (ditances between all pairs of cities)
dm = distmat(cities);

popsize = ncities;
pmut = 0.1;

o.verbose = 1;
o.MaxFunEvals = 1000;

% Call the evolutionary algorithm for the fPermTSP objective function.
[x, fx, hist] = minEA(...
    @fPermTSP, {dm}, ...
    @iniPermRandOrder, {popsize, ncities}, ...
    @selTournament, {popsize}, ...
    @xoverPermEdgeRecombination, {}, ...
    @mutPerm2Opt, {pmut}, ...
    @repTruncation, {}, ...
    o);
% Try 2 initializers:
% @iniPermRandOrder, {popsize, ncities}, ...
% @iniPermTSPNN, {popsize, dm}, ...

% Plot the history of the optimization
[ax,h1,h2] = plotyy(hist.evals, hist.fmin, hist.evals, hist.fmed, @steplogy, @steplogy);
xlabel('evaluations');
ylabel(ax(1),'loss function');
ylabel(ax(2),'# of nonimproving from last improving');

% Visualize the progress of optimization
visTSPHist(cities, hist);

% Save it
filename = sprintf('savEATSP%dCities%dEvals',ncities,o.MaxFunEvals);
save(filename);
