function x = xoverPermEdgeRecombination(xin, o)
%
% XOVERPERMEDGERECOMBINATION Edge recombnation crossover for permutation
% representation.
%

if ~exist('o','var') | isempty(o), o = struct; end
if iscell(o), o = struct(o{:}); end
if ~isfield(o,'verbose'), o.verbose = 0; end

if o.verbose,
    fprintf('Edge Recomb Xover');
end

[popsize, dim] = size(xin);
indpairs = ceil(popsize*rand(popsize,2));

x = zeros(popsize, dim);
for i = 1:popsize,
    x(i,:) = performXoverPermEdgeRecombination(xin(indpairs(i,1),:), xin(indpairs(i,2),:));
    if o.verbose, fprintf('.'); end
end

if o.verbose, fprintf('done.\n'); end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = performXoverPermEdgeRecombination(x1, x2)
% Create an array of connections for each node

if length(unique(x1)) ~= length(x1),
    error('SCP:xoverPermEdgeRecombination','Wrong input');
end
if length(unique(x2)) ~= length(x2),
    error('SCP:xoverPermEdgeRecombination','Wrong input');
end

cons = cell(length(x1),1);
for i = 1:length(x1)-1,
    cons{x1(i)} = [cons{x1(i)} x1(i+1)];
    cons{x1(i+1)} = [cons{x1(i+1)} x1(i)];
    cons{x2(i)} = [cons{x2(i)} x2(i+1)];
    cons{x2(i+1)} = [cons{x2(i+1)} x2(i)];
end
cons{x1(1)} = [cons{x1(1)} x1(end)];
cons{x1(end)} = [cons{x1(end)} x1(1)];
cons{x2(1)} = [cons{x2(1)} x2(end)];
cons{x2(end)} = [cons{x2(end)} x2(1)];

% Start creating the cycle in the first city of the first tour
x = zeros(1,length(x1));
x(1) = x1(1);

% Add cities one by one
for inode = 1:length(x1)-1,

    % First remove the just added city from the list of connections of all
    % other cities
    for i = 1:length(x1),
       cons{i} = setdiff2(cons{i}, x(inode));
    end
    
    % If the just added city has an unused connections...
    if ~isempty(cons{x(inode)}),
        nbhs = cons{x(inode)};
        lengths = zeros(length(nbhs),1);
        for inbh = 1:length(nbhs),
            lengths(inbh) = length(cons{nbhs(inbh)});
        end
        minnhood = min(lengths);
        imin = find(minnhood == lengths);
        rperm = randperm(length(imin));
        % ... select randomly one with the lowest number of connections
        x(inode+1) = nbhs(imin(rperm(1)));
    else
        % if no unused connections, choose randomly among the unused cities
        freenodes = setdiff2(x1,x);
        rperm = randperm(length(freenodes));
        x(inode+1) = freenodes(rperm(1));
    end

end

if length(unique(x)) ~= length(x),
    error('SCP:xoverPermEdgeRecombination','Wrong output');
end